@@ -5,7 +5,6 @@ email: zeng_bin8888@163.com
 create_dt: 2022/3/10 12:21
 describe: 人工智能模块，开发中，针对一些业内标准问题的研究流程
 """
-from . import utils
 from .utils import get_datetime_spans
-
+from .sk import evaluate_estimator, train_estimator
 
